#include "algorithm.h"


Algorithm::Algorithm(TriMesh& _mesh, QObject* _parent, const char* _name)
: QObject(_parent, _name),
  iter_count_user_(1),
  iter_count_(1), frame_(0), 
  mode_(0), n_modes_(0),
  mesh_(_mesh), in_animation_(false)
{
	modeBox_ = NULL;
    slider_ = NULL;
    playButton_ = NULL;
    toolBar_ = NULL;
    scrollView_ = NULL;

    if (_parent != NULL)
    {
        // create tool bar for algorithm widget
        toolBar_ = new QToolBar((QMainWindow*)_parent, "Tools");

        // choose settings for tool bar
        toolBar_->setCaption("Tools");
        toolBar_->setResizeEnabled(true);
        
        // dock tool bar on the right
        ((QMainWindow*)_parent)->moveDockWindow(toolBar_, Qt::Dock::Right);

        // generate scroll view
        scrollView_ = new QScrollView(toolBar_, "ScrollView");
        scrollView_->setFrameStyle(QFrame::NoFrame);
    }

	timer_ = new QTimer(0, "animateTimer");
	connect(timer_, SIGNAL(timeout()), SLOT(animate()));
}

Algorithm::~Algorithm(void)
{
    if (toolBar_ != NULL)
    {
        parent()->removeChild(toolBar_);
        delete toolBar_;
    }

    modeBox_ = NULL;
    slider_ = NULL;
    playButton_ = NULL;
}


void Algorithm::set_modeBox(QComboBox* _modeBox)
{
    modeBox_ = _modeBox;

    if (modeBox_ != NULL)
        modeBox_->insertItem(no_mode_.c_str());
}

void Algorithm::set_slider(QSlider* _slider)
{
    slider_ = _slider;

    if (slider_ != NULL)
        steps_ = slider_->pageStep();
}

void Algorithm::set_playButton(const QPushButton* _playButton)
{
    playButton_ = _playButton;
}

void Algorithm::add_mode(const std::string& _s)
{
	++n_modes_;
	mode_names_.push_back(_s);

    if (modeBox_ != NULL)
        modeBox_->insertItem(_s.c_str());
}

const std::string& Algorithm::draw_mode_name()
{
	return no_mode_;
}

unsigned int Algorithm::current_mode()
{
	return mode_ - 1;
}

const std::string& Algorithm::current_mode_name()
{
	return mode_ ? mode_names_[mode_-1] : no_mode_;
}

void Algorithm::set_mode(int _id)
{
	if(mode_ != _id)
	{
		mode_ = _id;
        if (modeBox_ != NULL)
            modeBox_->setCurrentItem(mode_);
		emit mode_changed(mode_);
	}
}

void Algorithm::set_iter_count(int _n)
{
	iter_count_user_ = _n;
}

void Algorithm::run()
{
	if(current_mode_name() == no_mode_)
	{
		QMessageBox::information(NULL, "Algorithm", "Please select a mode first.");
		return;
	}

    // set iteration count to the user input value
    iter_count_ = iter_count_user_;

	init();
	exec();
	postprocess();

	frame_ = 0;
    if (slider_ != NULL)
        slider_->setMaxValue(n_frames());
	set_frame(n_frames());

    // enable animation
    if (playButton_ != NULL)
        ((QWidget*)playButton_)->setEnabled(true);
}

void Algorithm::animate()
{
	if ((frame_ == n_frames()) && !in_animation_)
        set_frame(0);

    if(frame_ < n_frames())
	{
        in_animation_ = true;
		timer_->start(1000 / steps_, true);
		set_frame(frame_ + 1);
	}
    else
        in_animation_ = false;
}

void Algorithm::set_frame(int _n)
{
	_n = std::min(n_frames(), std::max(0, _n));     // should lie in [0..n_frames()]
	if(_n != frame_)
	{
		frame_ = _n;
        if (slider_ != NULL)
            slider_->setValue(frame_);
		apply();
		emit gl_changed();
		emit frame_changed(frame_);
	}
}


std::string Algorithm::no_mode_ = "";