#pragma once

// stl
#include <string>
#include <vector>
// qt
#include <qobject.h>
#include <qtoolbar.h>
#include <qdockarea.h>
#include <qmainwindow.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qscrollview.h>
//
#include "trimesh.h"

class QTimer;

// base class for an TriMesh Algorithm
class Algorithm :	public QObject
{
	Q_OBJECT

public:
    Algorithm(TriMesh& _mesh, QObject* _parent = 0, const char* _name = 0);
	virtual ~Algorithm(void);

protected:
    void set_modeBox(QComboBox* _modeBox);
    void set_slider(QSlider* _slider);
    void set_playButton(const QPushButton* _playButton);

	void add_mode(const std::string& _modeName);

	// purge animation
	virtual void init() {};
	// execute iteration steps
	// overwrite one of them in derived class
	virtual void exec() { for(int i = 0; i < iter_count_; ++i) step(); };
	virtual void step() {};
	// called after running the algorithm
	virtual void postprocess() {};
	// apply result to triangular mesh
	// overwrite one of them in derived class
	virtual void apply() 
	{ 
		apply(frame_ / steps_, static_cast<float>(frame_ % steps_) / steps_); 
		apply(static_cast<float>(frame_) / n_frames()); 
	}
	virtual void apply(int _iter, float _alpha) {};
	virtual void apply(float _alpha) {};

	template <typename T>
	T lerp(const T& _t0, const T& _t1, float _a)
	{
		return _t0 * (1.0f - _a) + _t1 * _a;
	}

public:
	virtual const std::string& draw_mode_name();

	unsigned int current_mode();
	const std::string& current_mode_name();

	// 0 is the first, n_frames() is the last frame
	int n_frames() const { return iter_count_ * steps_; }

public slots:
	// set some algorithm mode
	void set_mode(int _id);

	// set number of iterations of the algorithm
	void set_iter_count(int _n);

	// run the algorithm
	void run();
	// do ping-pong animation of the intermediate states
	void animate();
	// set the mesh to a intermediate state
	void set_frame(int _f);

signals:
	void mode_changed(int _id);
	void frame_changed(int _f);
	void gl_changed();

protected:
	int iter_count_user_, iter_count_, steps_, frame_;
    bool in_animation_;

	unsigned int              mode_;
	unsigned int              n_modes_;
	std::vector<std::string>  mode_names_;
	static std::string        no_mode_;

	QTimer* timer_;

    QComboBox* modeBox_;
    QSlider* slider_;
    const QPushButton* playButton_;

	TriMesh&      mesh_;

    QToolBar *toolBar_;                 // tool bar for algorithm widget
    QScrollView *scrollView_;           // scroll view for algorithm widget
};
