#pragma once

#include "algorithm.h"
#include "decimaterwidget.h"

// openmesh
#include <OpenMesh/Tools/Decimater/DecimaterT.hh>
#include <OpenMesh/Tools/Decimater/ModProgMeshT.hh>
#include <OpenMesh/Tools/Decimater/ModQuadricT.hh>
#include <OpenMesh/Tools/Decimater/ModRoundnessT.hh>
#include <OpenMesh/Tools/Decimater/ModNormalFlippingT.hh>
#include <OpenMesh/Tools/Decimater/ModIndependentSetsT.hh>

class Decimater : public Algorithm
{
    Q_OBJECT

	typedef OpenMesh::Decimater::DecimaterT<TriMesh>        DecimaterT;
	typedef OpenMesh::Decimater::ModProgMeshT<DecimaterT>   ModProgMesh;
    typedef OpenMesh::Decimater::ModQuadricT<DecimaterT>    ModQuadricT;
	typedef OpenMesh::Decimater::ModRoundnessT<DecimaterT>  ModRoundnessT;
    typedef OpenMesh::Decimater::ModNormalFlippingT<DecimaterT> ModNormalFlippingT;
    typedef OpenMesh::Decimater::ModIndependentSetsT<DecimaterT> ModIndependentSetsT;

	typedef OpenMesh::VPropHandleT<TriMesh::Point>      VPropPoint;
    typedef OpenMesh::VPropHandleT<TriMesh::TexCoord>   VPropTexCoord;

public:
	Decimater(TriMesh& _mesh, QObject* _parent = 0, const char* _name = 0);
	virtual ~Decimater();

public:
	virtual const std::string& draw_mode_name();

public slots:
    // set algorithm mode (overriding base function)
	void set_mode(int _id);
    // set percentage of edge collapses
	void set_edge_collapse(int _n);
    // set the mesh to a intermediate state (overriding base function)
	void set_frame(int _f);
    // animate the change between the intermediate states
	void delay_animate(void);
    // do ping-pong animation of the intermediate states (overriding base function)
	void animate(void);

    void set_mod_roundness(void);
    void set_mod_norm_flip(void);
    void set_mod_ind_sets(void);

protected:
	virtual void init();
    virtual void exec();
	virtual void apply(int _iter, float _alpha);

private:

	// do the collapses
	void decimate();

	static std::string draw_mode_;
	int cur_iter_;
    int edge_collapse_;

    int last_n_frame_;
    int last_frame_;

    bool no_animation_;

	DecimaterT *decimater_;
	ModProgMesh::Handle modPM_;

    bool mod_roundness_;                    // binary collapse constraint
    bool mod_norm_flip_;                    // binary collapse constraint
    bool mod_ind_sets_;                     // binary collapse constraint

    VPropPoint posInit_;
    std::vector<VPropPoint> posVec_;
    VPropTexCoord texInit_;
    std::vector<VPropTexCoord> texVec_;

    DecimaterWidget *decimaterWdg_;         // decimater widget
};

