/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include "subdivider.h"
#include "decimater.h"
#include "smoother.h"
#include "frequency.h"


// enumerate the different algorithms
enum enumAlgo
{
    eNone,
    eSubdivider,
    eSmoother,
    eDecimater,
    eFrequency,
};

void MainWindow::init()
{
    // do not allow toolbar docking on top and on bottom
    setDockEnabled(Qt::Dock::Top, false);
    setDockEnabled(Qt::Dock::Bottom, false);
    
    // no current algorithm selected
    cur_algo_ = eNone;

 renderWidget_ = new RenderWidget(this, "renderWidget");
    setCentralWidget(renderWidget_);

 mesh_ = &renderWidget_->mesh();

 algo_ = 0;
}

void MainWindow::destroy()
{
    if (algo_)
    {
 delete algo_;
             destroyAlgo();
    }
}

void MainWindow::fileNew()
{

}


void MainWindow::fileOpen()
{
    std::string ft = OpenMesh::IO::IOManager::Instance().qt_read_filters();

    const char* str = 0;
    str = ft.data();

    QString filter = str;

    QString s = QFileDialog::getOpenFileName(
     "data",
     filter,
     this,
     "open file dialog",
     "Choose a file" );

 if(s.isNull())
  return;

    if(algo_) delete algo_;

 if(!renderWidget_->open_mesh(s.ascii(), OpenMesh::IO::Options()))
 {
  QMessageBox::critical( this, "File Open", "Error reading '" + filename_ + "'.");  
  return;
 }

 setCaption(QFileInfo(filename_ = s).fileName().prepend("MeshDemo: "));
 
 resetAlgo();
}

void MainWindow::fileRevert()
{
 if(filename_.isNull())
  return;

    if(algo_) delete algo_;

 if(!renderWidget_->open_mesh(filename_.ascii(), OpenMesh::IO::Options()))
 {
  QMessageBox::critical( this, "File Revert", "Error reading '" + filename_ + "'.");  
  return;
 }
 
 resetAlgo();
}

void MainWindow::fileSave()
{
 if(filename_.isNull())
  return;

 if(!IO::write_mesh(*mesh_, filename_.ascii()))
 {
  QMessageBox::critical( this, "File Save", "Error writing '" + filename_ + "'.");  
  return;
 }
}


void MainWindow::fileSaveAs()
{
    std::string ft = OpenMesh::IO::IOManager::Instance().qt_write_filters();

    const char* str = 0;
    str = ft.data();

    QString filter = str;

    QString s = QFileDialog::getSaveFileName(
     "data", 
     filter,
     this,
     "save file dialog",
     "Choose a file" );

 if(s.isNull())
  return;

 if(!IO::write_mesh(*mesh_, s.ascii()))
 {
  QMessageBox::critical( this, "File Save", "Error writing '" + s + "'.");  
  return;
 }

 setCaption(QFileInfo(filename_ = s).fileName().prepend("MeshDemo: "));
}


void MainWindow::filePrint()
{

}


void MainWindow::helpIndex()
{

}


void MainWindow::helpContents()
{

}


void MainWindow::helpAbout()
{

}


void MainWindow::fileLoadTexture()
{
    QString s = QFileDialog::getOpenFileName(
     "data", 
     "Images (*.png *.bmp *xbm *.xpm *.pnm)",
     this,
     "load texture dialog",
     "Choose a texture image" );

    if(!s.isNull())
 renderWidget_->set_texture_file(s);
}


void MainWindow::fileExit()
{
 QApplication::exit(0);
}


void MainWindow::toolsDecimater()
{
    cur_algo_ = eDecimater;
    
    if(algo_) delete algo_;
 initAlgo(new ::Decimater(*mesh_, this, "decimater"));
 
    renderWidget_->set_texture_menu(true);
}


void MainWindow::toolsSubdivider()
{
    cur_algo_ = eSubdivider;
    
    if(algo_) delete algo_;
 initAlgo(new ::Subdivider(*mesh_, this, "subdivider"));
 
    renderWidget_->set_texture_menu(false);
}


void MainWindow::toolsSmoother()
{
    cur_algo_ = eSmoother;
    
    if(algo_) delete algo_;
 initAlgo(new Smoother(*mesh_, this, "smoother"));
 
    renderWidget_->set_texture_menu(true);
}


void MainWindow::toolsFrequency()
{
    cur_algo_ = eFrequency;
    
    if(algo_) delete algo_;
 initAlgo(new Frequency(*mesh_, this, "frequency"));
 
    renderWidget_->set_texture_menu(true);
}


void MainWindow::destroyAlgo()
{
 if(!algo_draw_mode_.find("CV:", 0, 1))
  renderWidget_->del_cv_mode(algo_draw_mode_.substr(3));
 else if(!algo_draw_mode_.find("VEC:", 0, 1))
  renderWidget_->del_vec_mode(algo_draw_mode_.substr(4));
 else
  renderWidget_->del_draw_mode(algo_draw_mode_);
 algo_ = 0;
}


void MainWindow::initAlgo(Algorithm* _algo)
{
 algo_ = _algo;
    connect(algo_, SIGNAL(destroyed()), SLOT(destroyAlgo()));
 connect(algo_, SIGNAL(gl_changed()), renderWidget_, SLOT(updateGL()));

 algo_draw_mode_ = algo_->draw_mode_name();
 if(!algo_draw_mode_.find("CV:", 0, 1))
  renderWidget_->add_cv_mode(algo_draw_mode_.substr(3));
 else if(!algo_draw_mode_.find("VEC:", 0, 1))
  renderWidget_->add_vec_mode(algo_draw_mode_.substr(4));
 else
  renderWidget_->add_draw_mode(algo_draw_mode_);
}



void MainWindow::resetAlgo()
{
    switch (cur_algo_)
    {
        case eNone:
        break;
        case eSubdivider:
            toolsSubdivider();
        break;
        case eSmoother:
            toolsSmoother();
        break;
        case eDecimater:
            toolsDecimater();
        break;
        case eFrequency:
            toolsFrequency();
        break;
    }
}
