#pragma once

// openmesh
#include <OpenMesh/Apps/QtViewer/MeshViewerWidgetT.hh>
// 
#include "trimesh.h"

typedef MeshViewerWidgetT<TriMesh> MeshViewerWidget;

class QStringList;

class RenderWidget : public MeshViewerWidget
{
	Q_OBJECT

public:

	typedef MeshViewerWidget Base;
	typedef RenderWidget     This;

public:

	RenderWidget(QWidget* _parent=0, const char* _name=0);
	virtual ~RenderWidget();

private:

	void init();

protected:

	virtual void draw_scene(const std::string& _draw_mode);

public:

	// inherited from MeshViewerWidgetT
	virtual bool open_mesh(const char* _filename, OpenMesh::IO::Options _opt);

	void add_cv_mode(const std::string& _s);
	void del_cv_mode(const std::string& _s);
	const std::string& current_cv_mode() const 
	{ return cv_mode_ ? cv_mode_names_[cv_mode_-1] : nomode_; }

	void add_vec_mode(const std::string& _s);
	void del_vec_mode(const std::string& _s);

    void set_texture_file(const char* _filename);
    void set_texture_menu(bool enable);

private slots:

	void slotCvSubmenu(int _id);    
	void slotVecSubmenu(int _id);    

private:

	QPopupMenu                *render_menu_;
    QPopupMenu                *cv_submenu_, *vec_submenu_;
	unsigned int              cv_mode_;
	unsigned int              n_cv_modes_, n_vec_modes_;
	std::vector<std::string>  cv_mode_names_, vec_mode_names_;
	static std::string        nomode_;

	std::string               tex_file_;            // texture file name

	bool show_tedges_;
};
