/*
 *  sort.h
 * 
 *  Author: Michael Sauter
 *
 *  Content: sort algorithms.
 *
 */

#ifndef _SORT_H_
#define _SORT_H_

#include <stdlib.h>
#include <assert.h>


/*======================================================================
  = Step of quicksort algorithm:                                       =
  =                                                                    =
  = index : array of indices to track permutations                     =
  = a     : array of the elements to sort                              =
  = l     : left array position                                        =
  = r     : right array position                                       =
  ======================================================================*/
template <typename IDX, typename T>
static inline void quicksort_step(IDX *index, T *a, int l, int r)
{
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//
    int i, j, m;
    IDX _idx;
    T item, v;
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//

    if (r > l)
    {
        m = (r + l) / 2;
        if (a[l] > a[r])
        {
            item = a[l]; _idx = index[l];
            a[l] = a[r]; index[l] = index[r];
            a[r] = item; index[r] = _idx;
        }
        if (a[l] > a[m])
        {
            item = a[l]; _idx = index[l];
            a[l] = a[m]; index[l] = index[m];
            a[m] = item; index[m] = _idx;
        }
        if (a[r] > a[m])
        {
            item = a[r]; _idx = index[r];
            a[r] = a[m]; index[r] = index[m];
            a[m] = item; index[m] = _idx;
        }

        i = l-1;
        j = r;
        v = a[r];

        while(true)
        {
            do{
                i++;
            } while(a[i] < v);

            do{
                j--;
            } while((a[j] > v) && (j > l));

            if (i >= j)
                break;

            item = a[i]; _idx = index[i];
            a[i] = a[j]; index[i] = index[j];
            a[j] = item; index[j] = _idx;
        }

        item = a[i]; _idx = index[i];
        a[i] = a[r]; index[i] = index[r];
        a[r] = item; index[r] = _idx;

        quicksort_step(index, a, l, (i-1));
        quicksort_step(index, a, (i+1), r);
    }
}


/*======================================================================
  = Quicksort: Sort array elements in ascending order and keep track   =
  =            of permutations                                         =
  =                                                                    =
  = index : array of indices to track permutations                     =
  = a     : array of the elements to sort                              =
  = n     : size of array                                              =
  ======================================================================*/
template <typename IDX, typename T>
void quicksort(IDX *index, T *a, int n)
{
    assert(index != NULL);
    assert(a != NULL);

    quicksort_step(index, a, 0, (n-1));
}


#endif      // _SORT_H_
