/*
 *  sparsematrix.cpp
 * 
 *  Authors: Michael Sauter
 *
 *  Content: class for a sparse matrix data structure
 *
 */

#include "sparsematrix.h"


SparseMatrix::SparseMatrix(void)
{
}


SparseMatrix::~SparseMatrix(void)
{
}


/*======================================================================
  = insert an element at (row, col) into matrix                        =
  = (no check whether such an element already exists!!)                =
  ======================================================================*/
void SparseMatrix::insert(int row, int col, double val)
{
    Mat_Entry entry;
    entry.row = row;
    entry.col = col;
    entry.val = val;

    matrix_.push_back(entry);
}


/*======================================================================
  = multiply matrix with vector                                        =
  ======================================================================*/
void SparseMatrix::mul_vec(int size, double *x, double *res)
{
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//
    sparse_mat_type::iterator iter;
    Mat_Entry entry;
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//

    // fill result vector with zeros
    memset(res, 0, size*sizeof(double));

    for(iter=matrix_.begin(); iter != matrix_.end(); iter++)
    {
        entry = (*iter);
        res[entry.row] += (entry.val * x[entry.col]);
    }
}


/*======================================================================
  = multiply matrix with matrix                                        =
  =                                                                    =
  = matrix has to be stored row-wise                                   =
  ======================================================================*/
void SparseMatrix::mul_mat(int size_n, int size_m, double *X, double *Res)
{
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//
    sparse_mat_type::iterator iter;
    Mat_Entry entry;
    int i;
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//

    // fill result vector with zeros
    memset(Res, 0, size_n*size_m*sizeof(double));

    for(iter=matrix_.begin(); iter != matrix_.end(); iter++)
    {
        for(i=0; i < size_m; i++)
        {
            entry = (*iter);
            Res[entry.row*size_m + i] += (entry.val * X[entry.col*size_m + i]);
        }
    }
}


/*======================================================================
  = multiply transposed matrix with vector                             =
  ======================================================================*/
void SparseMatrix::trans_mul_vec(int size, double *x, double *res)
{
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//
    sparse_mat_type::iterator iter;
    Mat_Entry entry;
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//

    // fill result vector with zeros
    memset(res, 0, size*sizeof(double));

    for(iter=matrix_.begin(); iter != matrix_.end(); iter++)
    {
        entry = (*iter);
        res[entry.col] += (entry.val * x[entry.row]);
    }
}


/*======================================================================
  = multiply transposed matrix with matrix                             =
  =                                                                    =
  = matrix has to be stored row-wise                                   =
  ======================================================================*/
void SparseMatrix::trans_mul_mat(int size_n, int size_m, double *X, double *Res)
{
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//
    sparse_mat_type::iterator iter;
    Mat_Entry entry;
    int i;
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -//

    // fill result vector with zeros
    memset(Res, 0, size_n*size_m*sizeof(double));

    for(iter=matrix_.begin(); iter != matrix_.end(); iter++)
    {
        for(i=0; i < size_m; i++)
        {
            entry = (*iter);
            Res[entry.col*size_m + i] += (entry.val * X[entry.row*size_m + i]);
        }
    }
}
