#pragma once

// 
#include "algorithm.h"
#include "subdividerwidget.h"
// stl
#include <limits>
// openmesh
#include <OpenMesh\Tools\Subdivider\Uniform\SubdividerT.hh>

class Subdivider : public Algorithm
{
	template <typename T>
	class Value
	{
	public:
		Value(const T& _t = invalid_value()) 
		: t_(_t) 
		{}

		bool valid() const 
		{ return t_ != invalid_value(); }

		const T& value() const 
		{ return t_; }

		void set_value(const T& _t) 
		{ t_ = _t; }

		static const T& invalid_value() 
		{ return inv_; }
	private:
		T t_;
		static const T inv_;
	};

	typedef OpenMesh::VPropHandleT<TriMesh::Point>  VPropPoint;
	typedef OpenMesh::VPropHandleT<Value<int> >     VPropIter;
	typedef OpenMesh::VPropHandleT<float>           VPropFloat;
	typedef OpenMesh::PropertyT<TriMesh::Point>     Points;

	typedef  OpenMesh::Subdivider::Uniform::SubdividerT<TriMesh> SubdividerT;

public:
	Subdivider(TriMesh& _mesh, QObject* _parent = 0, const char* _name = 0);
	virtual ~Subdivider();

public:
	virtual const std::string& draw_mode_name();

protected:
	virtual void init();
	virtual void step();
	virtual void postprocess();
	virtual void apply(int _iter, float _alpha);

private:

	void apply_topology();

    std::vector<VPropPoint> posVec;
	VPropIter  iterValue;
	VPropFloat iterFloat;
    
	SubdividerT *subdivider_;

	static std::string draw_mode_;
	int cur_iter_;
    int old_iter_count_;

    SubdividerWidget *subdividerWdg_;           // subdivider widget
};
