#pragma once

// stl
#include <string>
// qt
#include <qstringlist.h>
// openmesh
#include <OpenMesh/Core/Mesh/Types/TriMesh_ArrayKernelT.hh>
#include <OpenMesh/Core/Attributes/Attributes.hh>

struct TriMeshTraits : public OpenMesh::DefaultTraits
{
	VertexTraits
	{
	public:
		VertexT() 
		: first_(0.0f), second_(0.0f), tangent_(0.0f, 0.0f, 0.0f) 
		{}

		void set_first_curvature(const float& _first) { first_ = _first; }
		float first_curvature() const { return first_; }

		void set_second_curvature(const float& _second) { second_ = _second; }
		float second_curvature() const { return second_; }

		void set_first_tangent(const Point& _tangent) { tangent_ = _tangent; }
		const Point& first_tangent() const { return tangent_; }

		float mean_curvature() const { return (first_ + second_) / 2.0f; }
		float gaussian_curvature() const { return first_ * second_; }
		float shape_index() const { return -2.0 / M_PI * atan2f(first_ + second_, first_ - second_); }

	private:

		float first_, second_;
		Point tangent_;
	};

	VertexAttributes  ( OpenMesh::Attributes::Normal       |
						OpenMesh::Attributes::Status       );

	EdgeAttributes    ( OpenMesh::Attributes::Status       );

	HalfedgeAttributes( OpenMesh::Attributes::PrevHalfedge );

	FaceAttributes    ( OpenMesh::Attributes::Normal       |
						OpenMesh::Attributes::Status       );  
};

typedef OpenMesh::TriMesh_ArrayKernelT<TriMeshTraits> TriMeshBase;

class TriMesh : public TriMeshBase
{
	typedef OpenMesh::PropertyT<float>    VPropertyFloat;
	typedef OpenMesh::VPropHandleT<float> VPropHandleFloat;

	class Colorer
	{
	public:
		Colorer(TriMesh& _mesh) : mesh_(_mesh) {}
		Color operator()(const VertexHandle& _vh) const {};
	protected:
		TriMesh& mesh_;
	};
	class ConstColorer;
	class NormalColorer;
	class HueColorer;

	template<typename C>
	update_vertex_colors(const C& _colorer)
	{
		VertexIter vIt, vEnd = vertices_end();
		for(vIt = vertices_begin(); vIt != vEnd; ++vIt)
			set_color(vIt.handle(), _colorer(vIt.handle()));
	}

public:

	enum {NormalBit = 1, CurvatureBit = 2, ColorBit = 4};

	TriMesh();

    void init_vertex_texcoords();

	void update_vertex_curvatures();
	void update_vertex_colors();

	// tell trimesh that property is outdated
	void update_later(unsigned int _bits = ~0);

	// return the properties that are tagged for update
	unsigned int update_bits() const { return update_bits_; }

	// update outdated properties
	// @param _force update even if not tagged outdated
	void update_properties(unsigned int _bits = ~0, bool _force = false);

	void set_cv_mode(const std::string& _s);
	void set_cv_mode(VPropHandleFloat _p);

private:

	std::string         cv_mode_;
	static std::string  prop_cv_mode_;
	VPropHandleFloat    prop_cv_handle_;

	unsigned int update_bits_;
};
