//=============================================================================
//                                                                            
//                               OpenMesh                                     
//        Copyright (C) 2003 by Computer Graphics Group, RWTH Aachen          
//                           www.openmesh.org                                 
//                                                                            
//-----------------------------------------------------------------------------
//                                                                            
//                                License                                     
//                                                                            
//   This library is free software; you can redistribute it and/or modify it 
//   under the terms of the GNU Library General Public License as published  
//   by the Free Software Foundation, version 2.                             
//                                                                             
//   This library is distributed in the hope that it will be useful, but       
//   WITHOUT ANY WARRANTY; without even the implied warranty of                
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         
//   Library General Public License for more details.                          
//                                                                            
//   You should have received a copy of the GNU Library General Public         
//   License along with this library; if not, write to the Free Software       
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 
//                                                                            
//-----------------------------------------------------------------------------
//                                                                            
//   $Revision: 1.17 $
//   $Date: 2003/10/02 14:37:01 $
//                                                                            
//=============================================================================


#include <iostream>
#include <OpenMesh/Core/IO/MeshIO.hh> // include before kernel type!
#include <OpenMesh/Core/Mesh/Types/TriMesh_ArrayKernelT.hh>
#include <OpenMesh/Tools/Utils/Timer.hh>


using namespace OpenMesh;

struct MyTraits : public OpenMesh::DefaultTraits
{
  HalfedgeAttributes(OpenMesh::Attributes::PrevHalfedge);
};
typedef OpenMesh::TriMesh_ArrayKernelT<MyTraits>  MyMesh;


//-----------------------------------------------------------------------------


int main(int argc, char **argv)
{
  MyMesh  mesh;
  OpenMesh::Utils::Timer t;


  if (argc < 4) 
  {
    std::cerr << "Usage:  " << argv[0] << " n_iters  infile  outfile\n";
    exit(1);
  }


  // read mesh from argv[2]
  omout << "read mesh..." << std::flush;
  t.start();
  OpenMesh::IO::read_mesh(mesh, argv[2]);
  t.stop();
  omout << "done (" << t.as_string() << ")\n";



  // register vertex property storing cog data
  VPropHandleT<MyMesh::Point> cogs;
  mesh.add_property( cogs );



  // smoothing mesh argv[1] times
  MyMesh::VertexIter        v_it, v_end(mesh.vertices_end());
  MyMesh::VertexVertexIter  vv_it;
  MyMesh::Scalar            valence;
  unsigned int              i, N(atoi(argv[1]));

  omout << "smoothing..." << std::flush;
  t.start();

  for (i=0; i < N; ++i)
  {
    for (v_it=mesh.vertices_begin(); v_it!=v_end; ++v_it)
    {
      MyMesh::Point& cog = mesh.property(cogs, v_it.handle());

      cog[0] = cog[1] = cog[2] = valence = MyMesh::Scalar(0);

      for (vv_it=mesh.vv_iter(v_it); vv_it; ++vv_it)
      {
	cog += mesh.point(vv_it);
	++valence;
      }
      
      cog *= MyMesh::Scalar(1) / valence;
    }
    
    for (v_it=mesh.vertices_begin(); v_it!=v_end; ++v_it)
      if (!mesh.is_boundary(v_it))
	mesh.set_point(v_it, mesh.property(cogs, v_it));
  }

  t.stop();
  omout << "done (" << t.as_string() << ")\n";



  // write mesh to argv[3]
  omout << "write mesh..." << std::flush;
  t.start();
  OpenMesh::IO::write_mesh(mesh, argv[3]);
  t.stop();
  omout << "done (" << t.as_string() << ")\n";
}
